com.biffweb.tasks.impl.reload

Adapted from clojure.tools.namespace.repl. Licensed under EPL 1.0.

Adapted from clojure.tools.namespace.repl. Licensed under EPL 1.0.
raw docstring

full-reload-planclj

(full-reload-plan project-root directories)

Returns all the source files from this project in dependency order. Used by the soft-deploy task to evaluate files on the production server without that process needing to have c.t.n.r on the classpath: we compute the plan locally but execute it on the server.

Returns all the source files from this project in dependency order. Used by
the soft-deploy task to evaluate files on the production server without that
process needing to have c.t.n.r on the classpath: we compute the plan locally
but execute it on the server.
sourceraw docstring

refreshclj

(refresh)

Similar to c.t.n.r/refresh but doesn't unload namespaces first.

Similar to c.t.n.r/refresh but doesn't unload namespaces first.
sourceraw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries