Adapted from clojure.tools.namespace.repl. Licensed under EPL 1.0.
Adapted from clojure.tools.namespace.repl. Licensed under EPL 1.0.
(full-reload-plan project-root directories)Returns all the source files from this project in dependency order. Used by the soft-deploy task to evaluate files on the production server without that process needing to have c.t.n.r on the classpath: we compute the plan locally but execute it on the server.
Returns all the source files from this project in dependency order. Used by the soft-deploy task to evaluate files on the production server without that process needing to have c.t.n.r on the classpath: we compute the plan locally but execute it on the server.
(refresh)Similar to c.t.n.r/refresh but doesn't unload namespaces first.
Similar to c.t.n.r/refresh but doesn't unload namespaces first.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |